﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ClassLibraryGeometry
{
    /// <summary>
    /// Прямая на плоскости, проходящая через 
    /// две заданные точки
    /// Уравнение прямой:
    /// ax + by + c = 0 
    /// </summary>
    public class LineD
    {
        PointD point_A, point_B;
        double a, b, c;
        /// <summary>
        /// Конструктор по умолчанию
        /// Строит прямую, проходящую через точки
        /// (0, 0) и (1, 1)
        /// </summary>
        public LineD()
        {
            point_A = new PointD();
            point_B = new PointD(1, 1);
            EvalABC();
        }
        /// <summary>
        /// Конструктор 
        /// Строит прямую, проходящую через точки
        /// pt0 и pt1
        /// </summary>
        /// <param name="pt0">Точка прямой</param>
        /// <param name="pt1">Точка прямой</param>
        public LineD(PointD pt0, PointD pt1)
        {
            point_A = pt0; point_B = pt1;
            EvalABC();
        }
        /// <summary>
        /// Конструктор 
        /// Строит прямую, содержащую вектор 
        /// </summary>
        /// <param name="vector"> вектор</param>
        public LineD(VectorD vector)
        {
            point_A = vector.P0;
            point_B = vector.P1;
            EvalABC();
        }
        /// <summary>
        /// Вычисление коэффициентов уравнения прямой,
        /// проходящей через точки point_A и point_B 
        /// </summary>
        void EvalABC()
        {
            double x1 = point_A.X, y1 = point_A.Y;
            double x2 = point_B.X, y2 = point_B.Y;
            a = y1 - y2; b = x2 - x1; c = x1 * y2 - y1 * x2;
        }
        /// <summary>
        /// Метод-свойство
        /// </summary>
        public double A
        {
            get { return a; }
        }
        /// <summary>
        /// Метод-свойство
        /// </summary>
        public double B
        {
            get { return b; }
        }
        /// <summary>
        /// Метод-свойство
        /// </summary>
        public double C
        {
            get { return c; }
        }
        /// <summary>
        /// Параллельна ли прямая,
        /// заданная аргументом, исходной прямой 
        /// </summary>
        /// <param name="line1">прямая</param>
        /// <returns>
        /// true, если line1 параллельна исходной
        /// </returns>
        public bool IsParallel(LineD line1)
        {
            if (a == 0 && line1.a == 0) return true;
            if (b == 0 && line1.b == 0) return true;
            if (a / b == line1.a / line1.b) return true;
            return false;
        }
        /// <summary>
        /// точка пересечения прямой, заданной аргументом,
        /// и исходной прямой
        /// Предусловие: прямые пересекаются (не параллельны)
        /// </summary>
        /// <param name="line1">прямая</param>
        /// <returns>точка пересечения</returns>
        public PointD IntersectPoint(LineD line1)
        {
            double x = (b * line1.c - line1.b * c) / 
                (a * line1.b - line1.a * b);
            double y = (c * line1.a - line1.c * a) /
                (a * line1.b - line1.a * b);
            return new PointD(x, y);
        }



    }
}
