﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ClassLibraryGeometry
{
    /// <summary>
    /// class Polygons определяет многоугольник на плоскости,
    /// заданный множеством точек
    /// </summary>
    public class Polygon
    {
        // константы класса

        // поля класса

        /// <summary>
        /// n - число точек
        /// setPoints - исходное множество из n точек
        /// nfact - число "правильных" точек
        /// arrayPoints - массив из правильных точек множества
        /// Точки правильные для построения многоугольника,
        /// если нет повторений и 
        /// никакие три подряд идущие точки не лежат на одной прямой. 
        /// </summary>
        int n, nfact;
        PointD[] setPoints;
        PointD[] arrayPoints;

        // конструкторы класса
        /// <summary>
        /// Random конструктор
        /// Строит массив из n точек
        /// со случайными целочисленными координатами
        /// внутри квадрата со стороной Max
        /// </summary>
        /// <param name="n"></param>
        /// <param name="Max"></param>
        public Polygon(int n, int Max)
        {
            this.n = n; nfact = n;
            setPoints = new PointD[n];
            for (int i = 0; i < n; i++)
                setPoints[i] = new PointD(Max);
        }

        // процедуры-свойства класса

        // переопределенные методы родительского класса

        // собственные методы класса
        /// <summary>
        /// Построить по множеству точек setPoints
        /// массив правильных точек arrayPoints,
        /// определяющих многоугольник без пересечений сторон
        /// </summary>
        public void CreatePolygonFromSet()
        {
            //удалить повторяющиеся точки массива
            //вычислить nfact и arrayPoints
            RemoveEqualPoints();

            //Удалить из трех точек,
            //лежащих на одной прямой, среднюю точку
            RemoveMiddlePointOnLine();

            //отсортировать множество точек по координате X
            //для равных X сортировать по Y
            SortByXY();

            //Построить верхнюю и нижнюю цепочку точек, 
            //определяющих многоугольник
            FormHighAndLowChain();
        }
        /// <summary>
        /// Является ли построенный многоугольник выпуклым
        /// </summary>
        /// <returns>
        /// true - если выпуклый
        /// false - если нет        
        /// </returns>
        public bool IsConvex()
        {
            if (nfact < 3) return false;
            PointD p1, p2, p3;
            p1 = arrayPoints[0];
            p2 = arrayPoints[1];
            p3 = arrayPoints[2];
            double crossP0 = CrossProduct(p1, p2, p3);
            int s0 = Math.Sign(crossP0);
            double crossP;
            int s;
            for (int i = 1; i < nfact; i++)
            {
                p1 = arrayPoints[i];
                p2 = arrayPoints[(i + 1) % nfact];
                p3 = arrayPoints[(i + 2) % nfact];
                crossP = CrossProduct(p1, p2, p3);
                s = Math.Sign(crossP);
                if (s == 0) return false;
                if (s != s0) return false;
            }
            return true;
        }
        public PointD[] GetSetPoints()
        {
            return setPoints;
        }
        public PointD[] GetArrayPoints()
        {
            return arrayPoints;
        }

        // операции класса

        // закрытые методы класса

        /// <summary>
        /// функция сравнения двух точек по координате X
        /// </summary>
        /// <param name="item1">первая точка</param>
        /// <param name="item2">вторая точка</param>
        /// <returns>результат сравнения</returns>
        int CompareItemsByX(PointD item1, PointD item2)
        {
            if (item1.X < item2.X) return -1;
            else if (item1.X > item2.X) return 1;
            else //координаты по X совпадают
                // сравнение по Y (максимальные Y предшествуют)
                if (item1.Y < item2.Y) return 1;
                else if (item1.Y > item2.Y) return -1;
                else    //точки совпадают
                    return 0;
        }

        void SortByXY()
        {
           // SortedArray<PointD> sar = new SortedArray<PointD>();
           // sar.SimpleSort(CompareItemsByX);
            Array.Sort<PointD>(arrayPoints);
        }
        /// <summary>
        /// Удалить повторяющиеся точки массива
        /// вычислить nfact, arrayPoints
        /// </summary>
        void RemoveEqualPoints()
        {
            PointD[] temp = new PointD[n];
            PointD p;
            temp[0] = setPoints[0]; nfact = 1;
            for (int i = 1; i < n; i++)
            {
                p = setPoints[i];
                int j = 0;
                bool isEqual = false;
                while ((j < nfact) && !isEqual)
                {
                    isEqual = (p == temp[j]);
                    j++;
                }
                if (!isEqual)
                {
                    temp[nfact] = p;
                    nfact++;
                }
            }
            arrayPoints = new PointD[nfact];
            for (int i = 0; i < nfact; i++)
                arrayPoints[i] = temp[i];

        }
        void Swap(int i, int k)
        {
            PointD temp = new PointD();
            temp = setPoints[i];
            setPoints[i] = setPoints[k];
            setPoints[k] = temp;

        }

        void FormHighAndLowChain()
        {
            PointD[] temp = new PointD[nfact];
            temp[0] = arrayPoints[0];
            int k = 1;
            int i = 1;

            while (nfact - i >= 2)
            {
                if (arrayPoints[i].Y >= arrayPoints[i + 1].Y)
                {
                    temp[k] = arrayPoints[i];
                    temp[nfact - k] = arrayPoints[i + 1];
                }
                else
                {
                    temp[k] = arrayPoints[i + 1];
                    temp[nfact - k] = arrayPoints[i];
                }
                i += 2;
                k++;
            }
            if (nfact - i == 1)
                temp[k] = arrayPoints[i];
            arrayPoints = temp;
        }
        /// <summary>
        /// Удалить из трех точек,
        /// лежащих на одной прямой, среднюю точку
        /// </summary>
        void RemoveMiddlePointOnLine()
        {
            if (nfact > 2)
            {
                PointD p1, p2, p3;
                PointD[] temp = new PointD[nfact];
                temp[0] = arrayPoints[0];
                temp[1] = arrayPoints[1];
                int nt = 2;
                int res = 0;
                int j = 0;
                int k = 0;

                for (int i = 2; i < nfact; i++)
                {
                    p3 = arrayPoints[i];
                    j = 0;
                    res = 0;
                    while ((j < nt - 1) && (res == 0))
                    {
                        p1 = temp[j];
                        j++;
                        k = j;
                        while ((k < nt) && (res == 0))
                        {
                            p2 = temp[k];
                            k++;
                            res = Middle(p1, p2, p3);
                        }
                    }
                    switch (res)
                    {
                        case 0: //точки не на прямой
                            {
                                temp[nt] = p3;
                                nt++;
                                break;
                            }
                        case 1: //p1 - middle
                            {
                                temp[j - 1] = p3;
                                break;
                            }
                        case 2: //p2 - middle
                            {
                                temp[k - 1] = p3;
                                break;
                            }
                        case 3: //p3 - middle
                            {
                                break;
                            }
                    }

                }

                arrayPoints = new PointD[nt];
                for (int i = 0; i < nt; i++)
                    arrayPoints[i] = temp[i];
                nfact = nt;

            }

        }

        /// <summary>
        /// Расположение трех точек
        /// </summary>
        /// <param name="p1">первая точка</param>
        /// <param name="p2">вторая точка</param>
        /// <param name="p3">третья точка</param>
        /// <returns>
        /// 0 - если точки не лежат на одной прямой
        /// 1 - если точка 1 на прямой между точками 2 и 3
        /// 2 - если точка 2 на прямой между точками 1 и 3
        /// 3 - если точка 3 на прямой между точками 1 и 2
        /// </returns>
        int Middle(PointD p1, PointD p2, PointD p3)
        {

            double d1 = Distance(p1, p2);
            double d2 = Distance(p1, p3);
            double d3 = Distance(p3, p2);
            int mid = 0;
            if (d1 + d2 == d3)
                mid = 1;
            else
                if (d1 + d3 == d2)
                    mid = 2;
                else
                    if (d2 + d3 == d1)
                        mid = 3;
            return mid;
        }
        double Distance(PointD p1, PointD p2)
        {
            double a = p1.X - p2.X;
            double b = p1.Y - p2.Y;
            return Math.Sqrt(a * a + b * b);

        }
        /// <summary>
        /// векторное произведение векторов p1_p2, p2_p3,
        /// заданных точками. 
        /// </summary>
        /// <param name="p1">Начальная точка первого вектора</param>
        /// <param name="p2">Общая точка двух векторов</param>
        /// <param name="p3">Начальная точка второго вектора</param>
        /// <returns>векторное произведение</returns>
        double CrossProduct(PointD p1, PointD p2, PointD p3)
        {
            double r = (p1.X - p2.X) * (p3.Y - p2.Y) -
                        (p1.Y - p2.Y) * (p3.X - p2.X);
            return r;


        }
        /// <summary>
        /// Принадлежит ли точка, заданная аргументом,
        /// внутренней области многоугольника,
        /// включая его границы.
        /// Идея алгоритма: Из заданной точки проведем луч 
        /// (отрезок с длиной, равной максимальной стороне,
        /// Посчитаем число пересечений отрезка
        /// со сторонами многоугольника.
        /// Если оно нечетно, то точка внутри, иначе вне.
        /// Это правило не работает для вершин многоугольника.
        /// </summary>
        /// <param name="pt">точка</param>
        /// <returns>true, если внутри многоугольника</returns>
        public bool IsInside(PointD pt)
        {
            return true;
        }

        
    }

    class PolygonException : Exception
    {
        public PolygonException() { }
        public PolygonException(string message) : base(message) { }
        public PolygonException(string message, Exception e) :
            base(message, e) { }

    }
}
