﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ClassLibraryGeometry
{
    /// <summary>
    /// Вектор, заданный двумя точками
    /// </summary>
    public class VectorD
    {
        const string START_POINT =
            "Начальная точка вектора: ";
        const string FINISH_POINT =
            "Конечная точка вектора: ";
        const string NEW_LINE =
            "\r\n";
        /// <summary>
        /// Начальная точка
        /// </summary>
        PointD p0;
        /// <summary>
        /// Конечная точка
        /// </summary>
        PointD p1;

        //Конструкторы
        /// <summary>
        /// Конструктор
        /// Создает вектор с началом в (0, 0)
        /// и концом в (1,1)
        /// </summary>
        public VectorD()
        {
            p0 = new PointD();
            p1 = new PointD(1, 1);
        }
        /// <summary>
        /// Конструктор
        /// Создает вектор с началом в (0, 0)
        /// и концом в p
        /// </summary>
        /// <param name="p">Конечная точка</param>
        public VectorD(PointD p)
        {
            p0 = new PointD();
            p1 = p;
        }
        /// <summary>
        /// Конструктор
        /// Создает вектор с началом в p0
        /// и концом в p1
        /// </summary>
        /// <param name="p0">Начальная точка</param>
        /// <param name="p1">Конечная точка</param>
        public VectorD(PointD p0, PointD p1)
        {
            this.p0 = p0; this.p1 = p1;
        }

        //Методы - свойства
        /// <summary>
        /// Возвращает начальную точку вектора
        /// </summary>
        public PointD P0
        {
            get { return p0; }
        }
        /// <summary>
        /// Возвращает конечную точку вектора
        /// </summary>
        public PointD P1
        {
            get { return p1; }
        }

        public override string ToString()
        {
            string s1 = START_POINT + p0.ToString() + NEW_LINE;
            string s2 = FINISH_POINT + p1.ToString();
            return  s1+s2;
        }
        /// <summary>
        /// Пересекает ли вектор (отрезок), 
        /// заданный аргументом,
        /// искомый вектор.
        /// Предусловие: вектора не лежат на одной прямой
        /// Два вектора (отрезка) пересекаются, если
        /// пересекаются прямые, содержащие отрезки,
        /// точка пересечения прямых принадлежит обеим отрезкам
        /// </summary>
        /// <param name="vector1"> вектор </param>
        /// <returns>true, если вектора пересекаются</returns>
        public bool IntersectExist(VectorD vector1)
        {
            LineD line = new LineD(this);
            LineD line1 = new LineD(vector1);
            if (line.IsParallel(line1)) return false;
            PointD ptIntersect = line.IntersectPoint(line1);
            if (Between(ptIntersect) &&
                vector1.Between(ptIntersect))
                return true;
            else return false;
        }
        /// <summary>
        /// Принадлежит ли точка, заданная аргументом,
        /// вектору (отрезку)
        /// Предусловие: Точка принадлежит прямой,
        /// содержащей вектор (отрезок)
        /// </summary>
        /// <param name="pt">точка</param>
        /// <returns>true, если точка внутри отрезка</returns>
        bool Between(PointD pt)
        {
            double max, min;
            if (p0.X > p1.X) 
            { max = p0.X; min = p1.X;}
            else {max = p1.X; min = p0.X;}
            if (pt.X <= max && pt.X >= min) return true;
            else return false;
        }


    }
}
